@@ -9,7 +9,7 @@
 
 #include <qloggingcategory.h>
 
-static Q_LOGGING_CATEGORY(qLcAudioDecoder, "qt.multimedia.ffmpeg.audioDecoder")
+Q_STATIC_LOGGING_CATEGORY(qLcAudioDecoder, "qt.multimedia.ffmpeg.audioDecoder")
 
 QT_BEGIN_NAMESPACE
 
@@ -28,7 +28,7 @@ public:
             return {};
 
         if (!m_resampler)
-            m_resampler = std::make_unique<Resampler>(frame.codec(), m_format);
+            m_resampler = std::make_unique<QFFmpegResampler>(frame.codecContext(), m_format);
 
         emit newAudioBuffer(m_resampler->resample(frame.avFrame()));
 
@@ -40,7 +40,7 @@ signals:
 
 private:
     QAudioFormat m_format;
-    std::unique_ptr<Resampler> m_resampler;
+    std::unique_ptr<QFFmpegResampler> m_resampler;
 };
 
 class AudioDecoder : public PlaybackEngine
@@ -79,8 +79,7 @@ private:
     QPointer<Renderer> m_audioRenderer;
     QAudioFormat m_format;
 };
-}
-
+} // namespace QFFmpeg
 
 QFFmpegAudioDecoder::QFFmpegAudioDecoder(QAudioDecoder *parent)
     : QPlatformAudioDecoder(parent)
@@ -137,7 +136,20 @@ void QFFmpegAudioDecoder::start()
     connect(m_decoder.get(), &AudioDecoder::newAudioBuffer, this,
             &QFFmpegAudioDecoder::newAudioBuffer);
 
-    m_decoder->setMedia(m_url, m_sourceDevice);
+    QFFmpeg::MediaDataHolder::Maybe media = QFFmpeg::MediaDataHolder::create(m_url, m_sourceDevice, nullptr);
+
+    if (media) {
+        Q_ASSERT(media.value());
+        if (media.value()->streamInfo(QPlatformMediaPlayer::AudioStream).isEmpty())
+            error(QAudioDecoder::FormatError,
+                  QLatin1String("The media doesn't contain an audio stream"));
+        else
+            m_decoder->setMedia(std::move(*media.value()));
+    } else {
+        auto [code, description] = media.error();
+        errorSignal(code, description);
+    }
+
     if (!checkNoError())
         return;
 
@@ -149,7 +161,7 @@ void QFFmpegAudioDecoder::start()
     if (!checkNoError())
         return;
 
-    durationChanged(m_decoder->duration() / 1000);
+    durationChanged(QFFmpeg::toUserDuration(m_decoder->duration()).get());
     setIsDecoding(true);
 }
 
